#include "lstate.h"

/*
** $Id: lua.h,v 1.32 1999/05/11 20:29:19 roberto Exp $
** Lua - An Extensible Extension Language
** TeCGraf: Grupo de Tecnologia em Computacao Grafica, PUC-Rio, Brazil
** e-mail: lua@tecgraf.puc-rio.br
** www: http://www.tecgraf.puc-rio.br/lua/
** See Copyright Notice at the end of this file
*/


#ifndef lua_h
#define lua_h

#define LUA_VERSION	"Lua 3.2"
#define LUA_COPYRIGHT	"Copyright (C) 1994-1999 TeCGraf, PUC-Rio"
#define LUA_AUTHORS 	"W. Celes, R. Ierusalimschy & L. H. de Figueiredo"


#define LUA_NOOBJECT  0

#define LUA_ANYTAG (-1)

typedef struct lua_State lua_State;


typedef void (*lua_CFunction) (lua_State *L);
typedef unsigned int lua_Object;

lua_State            *lua_open (void);
void           lua_close (lua_State *L);
lua_State      *lua_setstate (lua_State *L, lua_State *st);

lua_Object     lua_settagmethod (lua_State *L, int tag, char *event); /* In: new method */
lua_Object     lua_gettagmethod (lua_State *L, int tag, char *event);

int            lua_newtag (lua_State *L);
int            lua_copytagmethods (lua_State *L, int tagto, int tagfrom);
void           lua_settag (lua_State *L, int tag); /* In: object */

void           lua_error (lua_State *L, char *s);
int            lua_dofile (lua_State *L, char *filename); /* Out: returns */
int            lua_dostring (lua_State *L, char *string); /* Out: returns */
int            lua_dobuffer (lua_State *L, char *buff, int size, char *name);
					  /* Out: returns */
int            lua_callfunction (lua_State *L, lua_Object f);
					  /* In: parameters; Out: returns */

void	       lua_beginblock (lua_State *L);
void	       lua_endblock (lua_State *L);

lua_Object     lua_lua2C (lua_State *L, int number);
#define	       lua_getparam(L, _)		lua_lua2C(L, _)
#define	       lua_getresult(L, _)		lua_lua2C(L, _)

int            lua_isnil (lua_State *L, lua_Object object);
int            lua_istable (lua_State *L, lua_Object object);
int            lua_isuserdata (lua_State *L, lua_Object object);
int            lua_iscfunction (lua_State *L, lua_Object object);
int            lua_isnumber (lua_State *L, lua_Object object);
int            lua_isstring (lua_State *L, lua_Object object);
int            lua_isfunction (lua_State *L, lua_Object object);

double         lua_getnumber (lua_State *L, lua_Object object);
char          *lua_getstring (lua_State *L, lua_Object object);
long           lua_strlen (lua_State *L, lua_Object object);
lua_CFunction  lua_getcfunction (lua_State *L, lua_Object object);
void	      *lua_getuserdata (lua_State *L, lua_Object object);


void 	       lua_pushnil (lua_State *L);
void           lua_pushnumber (lua_State *L, double n);
void           lua_pushlstring (lua_State *L, char *s, long len);
void           lua_pushstring (lua_State *L, char *s);
void           lua_pushcclosure (lua_State *L, lua_CFunction fn, int n);
void           lua_pushusertag (lua_State *L, void *u, int tag);
void           lua_pushobject (lua_State *L, lua_Object object);

lua_Object     lua_pop (lua_State *L);

lua_Object     lua_getglobal (lua_State *L, char *name);
lua_Object     lua_rawgetglobal (lua_State *L, char *name);
void           lua_setglobal (lua_State *L, char *name); /* In: value */
void           lua_rawsetglobal (lua_State *L, char *name); /* In: value */

void           lua_settable (lua_State *L); /* In: table, index, value */
void           lua_rawsettable (lua_State *L); /* In: table, index, value */
lua_Object     lua_gettable (lua_State *L); /* In: table, index */
lua_Object     lua_rawgettable (lua_State *L); /* In: table, index */

int            lua_tag (lua_State *L, lua_Object object);

char          *lua_nextvar (lua_State *L, char *varname);  /* Out: value */
int            lua_next (lua_State *L, lua_Object o, int i);
						/* Out: ref, value */ 

int            lua_ref (lua_State *L, int lock); /* In: value */
lua_Object     lua_getref (lua_State *L, int ref);
void	       lua_unref (lua_State *L, int ref);

lua_Object     lua_createtable (lua_State *L);

long	       lua_collectgarbage (lua_State *L, long limit);


/* =============================================================== */
/* some useful macros/functions */

#define lua_call(L, name)		lua_callfunction(L, lua_getglobal(L, name))

#define lua_pushref(L, ref)	lua_pushobject(L, lua_getref(L, ref))

#define lua_refobject(L, o,l)	(lua_pushobject(L, o), lua_ref(L, l))

#define lua_register(L, n,f)	(lua_pushcfunction(L, f), lua_setglobal(L, n))

#define lua_pushuserdata(L, u)     lua_pushusertag(L, u, 0)

#define lua_pushcfunction(L, f)	lua_pushcclosure(L, f, 0)

#define lua_clonetag(L, t)		lua_copytagmethods(L, lua_newtag(L), (t))

lua_Object     lua_seterrormethod (lua_State *L);  /* In: new method */

/* ==========================================================================
** for compatibility with old versions. Avoid using these macros/functions
** If your program does need any of these, define LUA_COMPAT2_5
*/


#ifdef LUA_COMPAT2_5


lua_Object     lua_setfallback (lua_State *L, char *event, lua_CFunction fallback);

#define lua_storeglobal		lua_setglobal
#define lua_type		lua_tag

#define lua_lockobject(L, o)  lua_refobject(L, o,1)
#define	lua_lock(L) lua_ref(L, 1)
#define lua_getlocked lua_getref
#define	lua_pushlocked lua_pushref
#define	lua_unlock lua_unref

#define lua_pushliteral(L, o)  lua_pushstring(L, o)

#define lua_getindexed(L, o,n) (lua_pushobject(L, o), lua_pushnumber(L, n), lua_gettable(L))
#define lua_getfield(L, o,f)   (lua_pushobject(L, o), lua_pushstring(L, f), lua_gettable(L))

#define lua_copystring(L, o) (strdup(L, lua_getstring(L, o)))

#define lua_getsubscript  lua_gettable
#define lua_storesubscript  lua_settable

#endif

#endif



/******************************************************************************
* Copyright (c) 1994-1999 TeCGraf, PUC-Rio.  All rights reserved.
* 
* Permission is hereby granted, without written agreement and without license
* or royalty fees, to use, copy, modify, and distribute this software and its
* documentation for any purpose, including commercial applications, subject to
* the following conditions:
* 
*  - The above copyright notice and this permission notice shall appear in all
*    copies or substantial portions of this software.
* 
*  - The origin of this software must not be misrepresented; you must not
*    claim that you wrote the original software. If you use this software in a
*    product, an acknowledgment in the product documentation would be greatly
*    appreciated (lua_State *L, but it is not required).
* 
*  - Altered source versions must be plainly marked as such, and must not be
*    misrepresented as being the original software.
*    
* The authors specifically disclaim any warranties, including, but not limited
* to, the implied warranties of merchantability and fitness for a particular
* purpose.  The software provided hereunder is on an "as is" basis, and the
* authors have no obligation to provide maintenance, support, updates,
* enhancements, or modifications.  In no event shall TeCGraf, PUC-Rio, or the
* authors be held liable to any party for direct, indirect, special,
* incidental, or consequential damages arising out of the use of this software
* and its documentation.
* 
* The Lua language and this implementation have been entirely designed and
* written by Waldemar Celes Filho, Roberto Ierusalimschy and
* Luiz Henrique de Figueiredo at TeCGraf, PUC-Rio.
*
* This implementation contains no third-party code.
******************************************************************************/
